/*******************************************************************************
 * Copyright (C) 2010  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 *******************************************************************************/
package org.ow2.orchestra.console.GWTConsole.client.DeployProcess;

import com.google.gwt.core.client.GWT;
import com.google.gwt.event.dom.client.ClickEvent;
import com.google.gwt.event.dom.client.ClickHandler;
import com.google.gwt.user.client.rpc.AsyncCallback;
import com.google.gwt.user.client.ui.FormPanel.SubmitCompleteEvent;
import com.google.gwt.user.client.ui.FormPanel.SubmitCompleteHandler;
import com.google.gwt.user.client.ui.RootPanel;

import org.ow2.orchestra.console.GWTConsole.client.ConsoleEventBus;
import org.ow2.orchestra.console.GWTConsole.client.OrchestraManagementException;
import org.ow2.orchestra.console.GWTConsole.client.OrchestraManagementServices;
import org.ow2.orchestra.console.GWTConsole.client.OrchestraManagementServicesAsync;
import org.ow2.orchestra.console.GWTConsole.client.Commons.PresenterIF;
import org.ow2.orchestra.console.GWTConsole.client.Event.ViewRefreshEvent;
import org.ow2.orchestra.console.GWTConsole.client.ListProcess.PListProcess;
import org.ow2.orchestra.console.GWTConsole.client.Popup.PPopup;
import org.ow2.orchestra.console.GWTConsole.client.Popup.PPopup.PopupType;
import org.ow2.orchestra.console.GWTConsole.client.Tab.PTab;


/**
 * Class that contains the presenter part of the DeployProcess module.
 *
 * @author Mathieu Crassin
 *
 */
public class PDeployProcess extends PresenterIF {

  /**
   * @author Mathieu Crassin
   *
   * The interface that represents the view
   *
   */
  protected VDeployProcessIF view;

  /**
   * @author Mathieu Crassin
   *
   * Instantiate the stub of the orchestra management server
   *
   */
  private final OrchestraManagementServicesAsync orchestramanagement = GWT.create(OrchestraManagementServices.class);

  /**
   * @author Mathieu Crassin
   *
   * Constant strings to parse the response of the post request.
   *
   */
  public static final String ENDLINE_STR = ";";
  public static final String MESSAGE_ERROR_STR = "Error: ";
  public static final String DETAILS_ERROR_STR = "Details: ";

  /**
   * @author Jean-Yves Goubet
   *
   * Presenter needed to ask a refresh of list process
   *
   */
  private final PListProcess pListProcess;


  /**
   * @author Mathieu Crassin
   *
   * Name of the different style.
   *
   */
  protected static final String NORMALAPPLISTYLE = "normalAppliStyle";
  protected static final String LOADINGAPPLISTYLE = "loadingAppliStyle";


  /**
   * @author Mathieu Crassin
   *
   * Application state, used when a bar is uploaded.
   *
   */
  public static enum ApplicationState {
    NORMAL,
    LOADING
  }


  /**
   * Constructor of the presenter
   *
   * @author Mathieu Crassin
   * @param dp : view of the deploy process
   * @param plp : presenter of the list process view
   *
   */
  public PDeployProcess(final VDeployProcess dp, final PListProcess plp) {
    //
    this.pListProcess = plp;
    //
    this.bindingView(dp);
    //
    this.setDebugID();
    //
    this.addListeners();
  }


  /**
   * Define an id for each widget.
   * This Id is used by selenium for the integration phase.
   *
   * @author Mathieu Crassin
   *
   */
  @Override
  protected void setDebugID() {
    this.view.getFileUpload().ensureDebugId("fileUploadFieldId");
    this.view.getUploadButton().ensureDebugId("uploadButtonId");
    this.view.getFormPanel().ensureDebugId("fileUploadFormId");
  }


  /**
   * Attach our interface of the view with the real class
   *
   * @author Mathieu Crassin
   * @param viewToBind : interface that represents the view to bind to its presenter
   *
   */
  @Override
  protected void bindingView(final Object viewToBind) {
    this.view = (VDeployProcessIF) viewToBind;
  }


  /**
   * Create the different listeners
   *
   * @author Mathieu Crassin
   *
   */
  @Override
  protected void addListeners() {
    // add the upload button listener
    this.addButtonListeners();
    // add the submit complete form listener
    this.addFormListeners();
  }


  /**
   * Create the button listeners
   *
   * @author Mathieu Crassin
   *
   */
  private void addButtonListeners() {
    this.view.getUploadButton().addClickHandler(new ClickHandler() {
      public void onClick(final ClickEvent arg0) {

        PDeployProcess.this.changeAppliState(ApplicationState.LOADING);
        if (PDeployProcess.this.view.getFileUpload().getFilename() != null
            && PDeployProcess.this.view.getFileUpload().getFilename().length() > 0) {
          PDeployProcess.this.view.getFormPanel().submit();
        } else {
          PDeployProcess.this.changeAppliState(ApplicationState.NORMAL);

          GWT.log("UploadButton click: Upload file component is empty !");
          PPopup.launch(PPopup.PopupType.ErrorPopup, "Upload file component is empty !", null);
        }
      }
    });
  }


  /**
   * Create the form listener
   *
   * @author Mathieu Crassin
   *
   */
  private void addFormListeners() {
    this.view.getFormPanel().addSubmitCompleteHandler(new SubmitCompleteHandler() {
      public void onSubmitComplete(final SubmitCompleteEvent arg0) {
        if (!PDeployProcess.this.fctOnSubmitComplete(arg0)) {
          PDeployProcess.this.changeAppliState(ApplicationState.NORMAL);
        }
      }
    });
  }


  /**
   * Function called by the form submit complete listener
   *
   * @author Mathieu Crassin
   * @param submitCompleteEvent : response of the post request, we know if there are error in
   *             parsing the string submitCompleteEvent.getResults
   * @return boolean : true if succeeded, false otherwise
   *
   */
  private boolean fctOnSubmitComplete(final SubmitCompleteEvent submitCompleteEvent) {
    //
    final String inputfile = this.view.getFileUpload().getFilename();

    GWT.log("onSubmitComplete: inputfile=" + inputfile);

    this.view.getFormPanel().reset();

    // check if there are some errors, and adapt the string result
    /*
     * string = <pre>filename</pre> if there are no errors
     * string = <pre>Error: ...;Details: ...;</pre> if there are errors
     */
    String res = submitCompleteEvent.getResults();
    if (this.isErrorMessages(res)) {

      // case of error
      GWT.log("onSubmitComplete: Unknown error " + res);

      // get out the pre markup
      res = this.deleteUselessMarkup(res);
      if (res == null) {
        GWT.log("onSubmitComplete: Error in result format !");
        PPopup.launch(PPopup.PopupType.ErrorPopup, "Fail while upload file ", "Error in result format !");
        return false;
      }

      // parse error string
      final StringBuffer errormessage = new StringBuffer(), errordetails = new StringBuffer();
      if (!this.parseErrorMessages(res, errormessage, errordetails)) {
        GWT.log("onSubmitComplete: Error in result format !");
        PPopup.launch(PPopup.PopupType.ErrorPopup, "Fail while upload file ", "Error in result format !");
        return false;
      }

      PPopup.launch(PPopup.PopupType.ErrorPopup, errormessage.toString(), errordetails.toString());
      return false;

    } else {
      // all is right, we have res = <pre>filenamewithpath</pre>
      res = this.deleteUselessMarkup(res);
      if (res == null) {
        GWT.log("onSubmitComplete: Error in result format !");
        PPopup.launch(PPopup.PopupType.ErrorPopup, "Fail while upload file ",
            "Error in result format, <pre> or </pre> markup not present !");
        return false;
      }

      // now, check if the string that we have represent a filename with a path
      final String filename = PDeployProcess.extractFileName(res);
      if (filename == null) {
        GWT.log("onSubmitComplete: Error in filename !");
        PPopup.launch(PPopup.PopupType.ErrorPopup, "Fail while upload file ",
            "Error in result format, filename not contains a path !");
        return false;
      }

      GWT.log("onSubmitComplete: " + res + " uploaded correctly !");

      // now, we have the name of the file to deploy on orchestra
      // we launch the operation
      this.callOrchestraServlet(res);

      return true;
    }

  }


  /**
   * Extract filename from the string.
   *
   * @author Mathieu Crassin
   * @param res : string that contain a filename with a path
   * @return String : return the only the filename
   *
   */
  protected static String extractFileName(final String res) {
    //
    if (res == null) {
      return null;
    }
    // first check that the filename string contain a path
    final int index = res.lastIndexOf("/");
    final int index2 = res.lastIndexOf("\\");
    if (index == -1 && index2 == -1) {
      GWT.log("onSubmitComplete: Result error (no path)");

      return null;
    } else {
      // then extract only the filename
      String filename = "";
      if (index != -1) {
        filename = res.substring(index + 1);
      } else {
        filename = res.substring(index2 + 1);
      }
      GWT.log("onSubmitComplete: filename=" + filename);

      return filename;
    }
  }


  /**
   * Call orchestra servlet to deploy file named in parameter.
   *
   * @author Mathieu Crassin
   * @param completefilename : name of the file
   *
   */
  private void callOrchestraServlet(final String completefilename) {
    //launch the deploy of the file on the orchestra server thanks to a RPC Call
    GWT.log("deployToOrchestra: launch with " + completefilename);
    this.orchestramanagement.deployToOrchestra(completefilename, new AsyncCallback<Void>() {

      public void onSuccess(final Void result) {
        // log success
        GWT.log("deployToOrchestra: onSuccess");
        GWT.log("deployToOrchestra: File deployed on orchestra");
        // display info on a label on client window
        PPopup.launch(PopupType.InfoPopup, "File deployed on orchestra", null);
        // refresh process list
        if (PDeployProcess.this.pListProcess != null) {
          ConsoleEventBus.getEventBus().fireEvent(new ViewRefreshEvent(null, PTab.Type.LIST_PROCESS_ACTIVITY, true));
        }
        // change cursor & enable upload button
        PDeployProcess.this.changeAppliState(ApplicationState.NORMAL);
      }

      public void onFailure(final Throwable caught) {
        // log fail
        GWT.log("deployToOrchestra: onFailure message=" + caught.getMessage());
        // check error type
        if (caught instanceof OrchestraManagementException) {
          PPopup.launch(PPopup.PopupType.ErrorPopup, caught.getMessage(), ((OrchestraManagementException)caught).getDetails());
        } else {
        // other exception detect
          PPopup.launch(PPopup.PopupType.ErrorPopup, "Fail while deploy file !", caught.getMessage());
        }
        // change cursor & enable upload button
        PDeployProcess.this.changeAppliState(ApplicationState.NORMAL);
      }
    });
  }


  /**
   * Check if a file name is correct
   *
   * @author Mathieu Crassin
   * @param filename : name of the file to validate
   * @return boolean : true or false whether or not the file name is valid
   *
   */
  protected final boolean validateUploadFile(final String filename) {
    return filename.endsWith(".bar");
  }


  /**
   * Change the state of the application. The cursor change and the upload button is disabled.
   *
   * @author Mathieu Crassin
   * @param state : new state of the application
   *
   */
  protected void changeAppliState(final ApplicationState state) {
    // mode normal
    if (state == ApplicationState.NORMAL) {
      RootPanel.get().removeStyleName(PDeployProcess.LOADINGAPPLISTYLE);
      RootPanel.get().addStyleName(PDeployProcess.NORMALAPPLISTYLE);
      this.view.getUploadButton().setEnabled(true);
    } else if (state == ApplicationState.LOADING) {
    // loading mode
      RootPanel.get().removeStyleName(PDeployProcess.NORMALAPPLISTYLE);
      RootPanel.get().addStyleName(PDeployProcess.LOADINGAPPLISTYLE);
      this.view.getUploadButton().setEnabled(false);
    }
  }


  /**
   * Detect if a string contains an error or not.
   *
   * @author Mathieu Crassin
   * @param inputString : string to parse
   * @return boolean : whether or not the string matches
   *
   */
  protected boolean isErrorMessages(final String inputString) {
    return inputString.contains(PDeployProcess.MESSAGE_ERROR_STR) && inputString.contains(PDeployProcess.ENDLINE_STR + PDeployProcess.DETAILS_ERROR_STR);
  }


  /**
   * Parse a string that contains an error messages, and return the 2 strings.
   *
   * @author Mathieu Crassin
   * @param inputString : string to parse
   * @param errormessage : out param. that indicate the main error message.
   * @param errordetails : out param. that indicate the details.
   * @return boolean :true or false whether or not the function succeed.
   *
   */
  public boolean parseErrorMessages(final String inputString, final StringBuffer errormessage, final StringBuffer errordetails) {
    if (errormessage == null || errordetails == null) {
      return false;
    }
    // empty the string buffer
    errormessage.delete(0, errormessage.length());
    errordetails.delete(0, errordetails.length());

    final int b = inputString.indexOf(PDeployProcess.MESSAGE_ERROR_STR);
    final int m = inputString.indexOf(PDeployProcess.ENDLINE_STR + PDeployProcess.DETAILS_ERROR_STR);
    if (b == -1 || m == -1) {
      return false;
    }

    // complete the string buffer
    errormessage.append(inputString.substring(b + PDeployProcess.MESSAGE_ERROR_STR.length(), m));
    errordetails.append(inputString.substring(m + (PDeployProcess.ENDLINE_STR + PDeployProcess.DETAILS_ERROR_STR).length(), inputString.length() - 1));

    // if the details string is empty, we turn is value to null, to avoid to have an empty details textaarea in popup
    if (errordetails.toString().equals(" ") || errordetails.toString().equals("")) {
      errordetails.delete(0, errordetails.length());
    }

    return true;
  }


  /**
   * Delete markup '<pre>' if exists, return null in error case.
   * Also delete 'pre' markup with parameters.
   *
   * @author Mathieu Crassin
   * @param inputString : string to parse.
   * @return String : the result.
   */
  public String deleteUselessMarkup(String inputString) {
    boolean fin = false;
    while (!fin) {
      final int markupbegin = inputString.indexOf("<");
      if (markupbegin != -1) {
        final int markupend = inputString.indexOf(">", markupbegin);
        if (markupend != -1) {
          String res = "";
          if (markupbegin > 0) {
            res = inputString.substring(0, markupbegin);
          }
          res += inputString.substring(markupend + ">".length());

          inputString = res;
        } else {
          fin = true;
        }
      } else {
        fin = true;
      }
    }

    return inputString;
  }

  @Override
  public void destroy() {
    // no operations on destroy

  }

}

